
BEGIN {
  unless ($ENV{AUTHOR_TESTING}) {
    require Test::More;
    Test::More::plan(skip_all => 'these tests are for testing by the author');
  }
}

use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::NoTabs 0.15

use Test::More 0.88;
use Test::NoTabs;

my @files = (
    'bin/cobalt2',
    'bin/cobalt2-dbdump',
    'bin/cobalt2-installer',
    'bin/cobalt2-plugin-installcf',
    'lib/Bot/Cobalt.pm',
    'lib/Bot/Cobalt/Common.pm',
    'lib/Bot/Cobalt/Conf.pm',
    'lib/Bot/Cobalt/Conf/File.pm',
    'lib/Bot/Cobalt/Conf/File/Channels.pm',
    'lib/Bot/Cobalt/Conf/File/Core.pm',
    'lib/Bot/Cobalt/Conf/File/PerPlugin.pm',
    'lib/Bot/Cobalt/Conf/File/Plugins.pm',
    'lib/Bot/Cobalt/Conf/Role/Reader.pm',
    'lib/Bot/Cobalt/Core.pm',
    'lib/Bot/Cobalt/Core/ContextMeta.pm',
    'lib/Bot/Cobalt/Core/ContextMeta/Auth.pm',
    'lib/Bot/Cobalt/Core/ContextMeta/Ignore.pm',
    'lib/Bot/Cobalt/Core/Loader.pm',
    'lib/Bot/Cobalt/Core/Role/EasyAccessors.pm',
    'lib/Bot/Cobalt/Core/Role/IRC.pm',
    'lib/Bot/Cobalt/Core/Role/Singleton.pm',
    'lib/Bot/Cobalt/Core/Role/Timers.pm',
    'lib/Bot/Cobalt/Core/Sugar.pm',
    'lib/Bot/Cobalt/DB.pm',
    'lib/Bot/Cobalt/Error.pm',
    'lib/Bot/Cobalt/Frontend/RC.pm',
    'lib/Bot/Cobalt/Frontend/Utils.pm',
    'lib/Bot/Cobalt/IRC.pm',
    'lib/Bot/Cobalt/IRC/Event.pm',
    'lib/Bot/Cobalt/IRC/Event/Channel.pm',
    'lib/Bot/Cobalt/IRC/Event/Kick.pm',
    'lib/Bot/Cobalt/IRC/Event/Mode.pm',
    'lib/Bot/Cobalt/IRC/Event/Nick.pm',
    'lib/Bot/Cobalt/IRC/Event/Quit.pm',
    'lib/Bot/Cobalt/IRC/Event/Topic.pm',
    'lib/Bot/Cobalt/IRC/FloodChk.pm',
    'lib/Bot/Cobalt/IRC/Message.pm',
    'lib/Bot/Cobalt/IRC/Message/Public.pm',
    'lib/Bot/Cobalt/IRC/Role/AdminCmds.pm',
    'lib/Bot/Cobalt/IRC/Role/UserEvents.pm',
    'lib/Bot/Cobalt/IRC/Server.pm',
    'lib/Bot/Cobalt/Lang.pm',
    'lib/Bot/Cobalt/Logger.pm',
    'lib/Bot/Cobalt/Logger/Output.pm',
    'lib/Bot/Cobalt/Logger/Output/File.pm',
    'lib/Bot/Cobalt/Logger/Output/Term.pm',
    'lib/Bot/Cobalt/Manual/Plugins.pod',
    'lib/Bot/Cobalt/Manual/Plugins/Config.pod',
    'lib/Bot/Cobalt/Manual/Plugins/Dist.pod',
    'lib/Bot/Cobalt/Manual/Plugins/Tutorial.pod',
    'lib/Bot/Cobalt/Plugin/Alarmclock.pm',
    'lib/Bot/Cobalt/Plugin/Auth.pm',
    'lib/Bot/Cobalt/Plugin/Extras/CPAN.pm',
    'lib/Bot/Cobalt/Plugin/Extras/DNS.pm',
    'lib/Bot/Cobalt/Plugin/Extras/Debug.pm',
    'lib/Bot/Cobalt/Plugin/Extras/Karma.pm',
    'lib/Bot/Cobalt/Plugin/Extras/Money.pm',
    'lib/Bot/Cobalt/Plugin/Extras/Relay.pm',
    'lib/Bot/Cobalt/Plugin/Extras/TempConv.pm',
    'lib/Bot/Cobalt/Plugin/Games.pm',
    'lib/Bot/Cobalt/Plugin/Games/Dice.pm',
    'lib/Bot/Cobalt/Plugin/Games/Magic8.pm',
    'lib/Bot/Cobalt/Plugin/Games/RockPaperScissors.pm',
    'lib/Bot/Cobalt/Plugin/Games/Roulette.pm',
    'lib/Bot/Cobalt/Plugin/Info3.pm',
    'lib/Bot/Cobalt/Plugin/Master.pm',
    'lib/Bot/Cobalt/Plugin/OutputFilters/StripColor.pm',
    'lib/Bot/Cobalt/Plugin/OutputFilters/StripFormat.pm',
    'lib/Bot/Cobalt/Plugin/PluginMgr.pm',
    'lib/Bot/Cobalt/Plugin/RDB.pm',
    'lib/Bot/Cobalt/Plugin/RDB/AsyncSearch.pm',
    'lib/Bot/Cobalt/Plugin/RDB/AsyncSearch/Worker.pm',
    'lib/Bot/Cobalt/Plugin/RDB/Database.pm',
    'lib/Bot/Cobalt/Plugin/RDB/SearchCache.pm',
    'lib/Bot/Cobalt/Plugin/Rehash.pm',
    'lib/Bot/Cobalt/Plugin/Seen.pm',
    'lib/Bot/Cobalt/Plugin/Version.pm',
    'lib/Bot/Cobalt/Plugin/WWW.pm',
    'lib/Bot/Cobalt/Serializer.pm',
    'lib/Bot/Cobalt/Timer.pm',
    'lib/Bot/Cobalt/Utils.pm',
    't/00-report-prereqs.dd',
    't/00-report-prereqs.t',
    't/01_utils/01_mkpasswd.t',
    't/01_utils/02_rplprintf.t',
    't/01_utils/03_timestr.t',
    't/01_utils/04_glob.t',
    't/01_utils/05_color.t',
    't/01_utils/06_frontend.t',
    't/01_utils/07_rcfiles.t',
    't/01_utils/08_error.t',
    't/01_utils/09_serial_freezethaw.t',
    't/01_utils/10_serial_file.t',
    't/02_logger/01_output_file.t',
    't/02_logger/02_output_term.t',
    't/02_logger/03_output.t',
    't/02_logger/04_logger.t',
    't/03_conf/01_file.t',
    't/03_conf/02_f_cobalt.t',
    't/03_conf/03_f_channels.t',
    't/03_conf/04_f_plugins.t',
    't/03_conf/05_conf.t',
    't/04_core/01_flood.t',
    't/04_core/02_lang.t',
    't/04_core/03_contextmeta.t',
    't/04_core/04_meta_ignore.t',
    't/04_core/05_meta_auth.t',
    't/04_core/06_timer.t',
    't/04_core/07_core_basic.t',
    't/05_irc/01_loadable.t',
    't/05_irc/02_irc_server.t',
    't/05_irc/03_irc_event.t',
    't/05_irc/04_irc_message.t',
    't/05_irc/05_irc_event_channel.t',
    't/05_irc/06_irc_event_kick.t',
    't/05_irc/07_irc_event_mode.t',
    't/05_irc/08_irc_event_nick.t',
    't/05_irc/09_irc_event_topic.t',
    't/05_irc/10_irc_event_quit.t',
    't/06_db/01_db.t',
    't/06_db/02_dbdump.t',
    't/07_plugins/01_loadable_coreset.t',
    't/07_plugins/02_with_loader.t',
    't/07_plugins/03_rdb_searchcache.t',
    't/07_plugins/04_rdb_database.t',
    't/08_cmds/01_cobalt2.t',
    't/08_cmds/02_installer.t',
    't/08_cmds/03_dbdump.t',
    't/author-no-tabs.t',
    't/author-pod-syntax.t',
    't/release-cpan-changes.t',
    't/release-dist-manifest.t',
    't/release-pod-linkcheck.t'
);

notabs_ok($_) foreach @files;
done_testing;
